-- 	KuhnDLCAddin Script
--
-- 	Author: 	Silas770
-- 	Date:		27.5.2017
--		Version:	1.0 (LS17)
--
--	Contact at NKB Modding:
--	https://nkb-modding.com/index.php/User/6877-Silas770/
-- Contact at Vertex Dezign:
--	http://vertexdezign.net/forum/index.php/User/2081-Silas770/
--

KuhnDLCAddin = {};
KuhnDLCAddin.modDir = g_currentModDirectory;

function KuhnDLCAddin.prerequisitesPresent(specializations)
	return true;
end;

function KuhnDLCAddin:load(savegame)
	self.createWorkArea = KuhnDLCAddin.createWorkArea;
	self.createTestArea = KuhnDLCAddin.createTestArea;
	self.createSection = KuhnDLCAddin.createSection;
	self.createAnimation = KuhnDLCAddin.createAnimation;
	
	if self.configFileName == getUserProfileAppPath().."pdlc/kuhnPack/vehicles/tools/kuhn/kuhnDeltis1302.xml" then
		self.getIsFoldAllowed = Utils.overwrittenFunction(self.getIsFoldAllowed, self.getIsFoldAllowedSC);
		self.processSprayerAreas = Utils.overwrittenFunction(self.processSprayerAreas, self.processSprayerAreasSC);
		VehicleCamera.mouseEvent = Utils.overwrittenFunction(VehicleCamera.mouseEvent, self.overwrittenMouseEventSC);
		
		self.sectionControlActive = true;
		self.changeModeButton = "IMPLEMENT_EXTRA3";
		self.deactivateSound = true;
		self.allowFoldIfDeactivated = true;
		if self.modes == nil then
			self.modes = {};
		end;
		table.insert(self.modes, self:getModeByName("singleMode"), {enabled = true, g_i18nName = "singleMode"});
		table.insert(self.modes, self:getModeByName("leftToRightMode"), {enabled = true, g_i18nName = "leftToRightMode"});
		table.insert(self.modes, self:getModeByName("rightToLeftMode"), {enabled = true, g_i18nName = "rightToLeftMode"});
		table.insert(self.modes, self:getModeByName("automaticMode"), {enabled = true, g_i18nName = "automaticMode"});
		
		-- workAreas
		self.workAreaBackup = self.workAreas[1];
		self.workAreas[1] = nil;
		self.workAreaByType[WorkArea.AREATYPE_SPRAYER][1] = nil;
		
		local workAreasParentIndex = Utils.indexToObject(self.components, "0>0|2");
		local workAreaData = {	{sx = 10.5, sy = 0, sz = -0.6, wx = 10.5, wy = 0, wz = -1.4, hx = 7.535, hy = 0, hz = -0.6},			-- workArea left 1
												{sx = 7.535, sy = 0, sz = -0.6, wx = 7.535, wy = 0, wz = -1.4, hx = 4.085, hy = 0, hz = -0.6},		-- workArea left 2
												{sx = 4.085, sy = 0, sz = -0.6, wx = 4.085, wy = 0, wz = -1.4, hx = 1.014, hy = 0, hz = -0.6},		-- workArea left 3
												{sx = 1.014, sy = 0, sz = -0.6, wx = 1.014, wy = 0, wz = -1.4, hx = -1.014, hy = 0, hz = -0.6},	-- workArea middle
												{sx = -1.014, sy = 0, sz = -0.6, wx = -1.014, wy = 0, wz = -1.4, hx = -4.085, hy = 0, hz = -0.6},	-- workArea right 3
												{sx = -4.085, sy = 0, sz = -0.6, wx = -4.085, wy = 0, wz = -1.4, hx = -7.535, hy = 0, hz = -0.6},	-- workArea right 2
												{sx = -7.535, sy = 0, sz = -0.6, wx = -7.535, wy = 0, wz = -1.4, hx = -10.5, hy = 0, hz = -0.6}};	-- workArea right 1							
		for index, data in pairs(workAreaData) do
			table.insert(self.workAreas, index, self:createWorkArea(data.sx, data.sy, data.sz, data.wx, data.wy, data.wz, data.hx, data.hy, data.hz, workAreasParentIndex));
		end;
		
	    for _, area in pairs(self.workAreas) do
	        if self.workAreaByType[area.type] == nil then
	            self.workAreaByType[area.type] = {};
	        end;
	        table.insert(self.workAreaByType[area.type], area);
	    end;
		
		-- animations
		local animationParts = {	{name = "leftSection1", parts = {"0>0|1|0|0|0|0|0|0|5", "0>0|1|0|0|0|0|0|0|0|2"}}, 
												{name = "leftSection2", parts = {"0>0|1|0|0|0|0|0|5"}}, 
												{name = "leftSection3", parts = {"0>0|1|0|0|0|0|7"}}, 
												{name = "middleSection", parts = {"0>0|1|0|0|7"}}, 
												{name = "rightSection3", parts = {"0>0|1|0|0|1|0|7"}}, 
												{name = "rightSection2", parts = {"0>0|1|0|0|1|0|0|5"}}, 
												{name = "rightSection1", parts = {"0>0|1|0|0|1|0|0|0|5", "0>0|1|0|0|1|0|0|0|0|2"}}};							
		for _, animationPart in pairs(animationParts) do
			self:createAnimation(animationPart.name, animationPart.parts);
		end;
		 
		-- sections and testAreas
		if self.sections == nil then
			self.sections = {};
		end;
		local sectionData = {	{animName = "leftSection1", inputName = "leftSection2Fold", workAreas = {1}, width = 2.965}, 
											{animName = "leftSection2", inputName = "leftSection3Fold", workAreas = {2}, width = 3.45}, 
											{animName = "leftSection3", inputName = "leftSection4Fold", workAreas = {3}, width = 3.071}, 
											{animName = "middleSection", inputName = "middleSectionFold", workAreas = {4}, width = 2.028}, 
											{animName = "rightSection3", inputName = "rightSection4Fold", workAreas = {5}, width = 3.071}, 
											{animName = "rightSection2", inputName = "rightSection3Fold", workAreas = {6}, width = 3.45}, 
											{animName = "rightSection1", inputName = "rightSection2Fold", workAreas = {7}, width = 2.965}};
		local testAreasParentIndex = Utils.indexToObject(self.components, "0>0");
		local testAreaData = {{sx = 10.5, sy = 0, sz = 0.8, wx = 10.5, wy = 0, wz = -0.6, hx = 7.535, hy = 0, hz = 0.8},			-- testArea left 1
											{sx = 7.535, sy = 0, sz = 0.8, wx = 7.535, wy = 0, wz = -0.6, hx = 4.085, hy = 0, hz = 0.8},		-- testArea left 2
											{sx = 4.085, sy = 0, sz = 0.8, wx = 4.085, wy = 0, wz = -0.6, hx = 1.014, hy = 0, hz = 0.8},		-- testArea left 3
											{sx = 1.014, sy = 0, sz = 0.8, wx = 1.014, wy = 0, wz = -0.6, hx = -1.014, hy = 0, hz = 0.8},		-- testArea middle
											{sx = -1.014, sy = 0, sz = 0.8, wx = -1.014, wy = 0, wz = -0.6, hx = -4.085, hy = 0, hz = 0.8},	-- testArea right 3
											{sx = -4.085, sy = 0, sz = 0.8, wx = -4.085, wy = 0, wz = -0.6, hx = -7.535, hy = 0, hz = 0.8},	-- testArea right 2
											{sx = -7.535, sy = 0, sz = 0.8, wx = -7.535, wy = 0, wz = -0.6, hx = -10.5, hy = 0, hz = 0.8}};	-- testArea right 1
											
		for index, data in pairs(sectionData) do
			local testArea = nil;
			if testAreaData[index] ~= nil then
				local tAData = testAreaData[index];
				testArea = self:createTestArea(tAData.sx, tAData.sy, tAData.sz, tAData.wx, tAData.wy, tAData.wz, tAData.hx, tAData.hy, tAData.hz, testAreasParentIndex);
			end;
			table.insert(self.sections, index, self:createSection(data.animName, data.inputName, data.workAreas, data.width, testArea));
		end;
		
		self:createSCGUI();
	elseif self.configFileName == getUserProfileAppPath().."pdlc/kuhnPack/vehicles/trailers/kuhn/kuhnMetris4102.xml" then
		self.getIsFoldAllowed = Utils.overwrittenFunction(self.getIsFoldAllowed, self.getIsFoldAllowedSC);
		self.processSprayerAreas = Utils.overwrittenFunction(self.processSprayerAreas, self.processSprayerAreasSC);
		
		self.sectionControlActive = true;
		self.changeModeButton = "IMPLEMENT_EXTRA3";
		self.deactivateSound = true;
		self.allowFoldIfDeactivated = true;
		if self.modes == nil then
			self.modes = {};
		end;
		table.insert(self.modes, self:getModeByName("singleMode"), {enabled = true, g_i18nName = "singleMode"});
		table.insert(self.modes, self:getModeByName("leftToRightMode"), {enabled = true, g_i18nName = "leftToRightMode"});
		table.insert(self.modes, self:getModeByName("rightToLeftMode"), {enabled = true, g_i18nName = "rightToLeftMode"});
		table.insert(self.modes, self:getModeByName("automaticMode"), {enabled = true, g_i18nName = "automaticMode"});
		
		-- workAreas
		self.workAreaBackup = self.workAreas[1];
		self.workAreas[1] = nil;
		self.workAreaByType[WorkArea.AREATYPE_SPRAYER][1] = nil;
		
		local workAreasParentIndex = Utils.indexToObject(self.components, "0>0|3");
		local workAreaData = {	{sx = 18, sy = 0, sz = -2.831, wx = 18, wy = 0, wz = -3.831, hx = 15.492, hy = 0, hz = -2.831},						-- workArea left 1
												{sx = 15.492, sy = 0, sz = -2.831, wx = 15.492, wy = 0, wz = -3.831, hx = 11.995, hy = 0, hz = -2.831},		-- workArea left 2
												{sx = 11.995, sy = 0, sz = -2.831, wx = 11.995, wy = 0, wz = -3.831, hx = 5.966, hy = 0, hz = -2.831},			-- workArea left 3
												{sx = 5.966, sy = 0, sz = -2.831, wx = 5.966, wy = 0, wz = -3.831, hx = 0.863, hy = 0, hz = -2.831},				-- workArea left 4
												{sx = 0.863, sy = 0, sz = -3.37, wx = 0.863, wy = 0, wz = -4.039, hx = -0.863, hy = 0, hz = -3.37},				-- workArea middle
												{sx = -0.863, sy = 0, sz = -2.831, wx = -0.863, wy = 0, wz = -3.831, hx = -5.966, hy = 0, hz = -2.831},			-- workArea right 4
												{sx = -5.966, sy = 0, sz = -2.831, wx = -5.966, wy = 0, wz = -3.831, hx = -11.995, hy = 0, hz = -2.831},		-- workArea right 3
												{sx = -11.995, sy = 0, sz = -2.831, wx = -11.995, wy = 0, wz = -3.831, hx = -15.492, hy = 0, hz = -2.831},	-- workArea right 2
												{sx = -15.492, sy = 0, sz = -2.831, wx = -15.492, wy = 0, wz = -3.831, hx = -18, hy = 0, hz = -2.831}};			-- workArea right 1							
		for index, data in pairs(workAreaData) do
			table.insert(self.workAreas, index, self:createWorkArea(data.sx, data.sy, data.sz, data.wx, data.wy, data.wz, data.hx, data.hy, data.hz, workAreasParentIndex));
		end;
		
	    for _, area in pairs(self.workAreas) do
	        if self.workAreaByType[area.type] == nil then
	            self.workAreaByType[area.type] = {};
	        end;
	        table.insert(self.workAreaByType[area.type], area);
	    end;
		
		-- animations
		local animationParts = {	{name = "leftSection1", parts = {"0>0|2|0|0|0|0|0|0|0|0|0|1"}}, 
												{name = "leftSection2", parts = {"0>0|2|0|0|0|0|0|0|0|0|4"}}, 
												{name = "leftSection3", parts = {"0>0|2|0|0|0|0|0|0|6"}}, 
												{name = "leftSection4", parts = {"0>0|2|0|0|0|0|5"}}, 
												{name = "middleSection", parts = {"0>0|2|0|0|8"}}, 
												{name = "rightSection4", parts = {"0>0|2|0|0|1|0|5"}}, 
												{name = "rightSection3", parts = {"0>0|2|0|0|1|0|0|0|6"}}, 
												{name = "rightSection2", parts = {"0>0|2|0|0|1|0|0|0|0|0|4"}}, 
												{name = "rightSection1", parts = {"0>0|2|0|0|1|0|0|0|0|0|0|1"}}};							
		for _, animationPart in pairs(animationParts) do
			self:createAnimation(animationPart.name, animationPart.parts);
		end;
		 
		-- sections and testAreas
		if self.sections == nil then
			self.sections = {};
		end;
		local sectionData = {	{animName = "leftSection1", inputName = "leftSection1Fold", workAreas = {1}, width = 2.508}, 
											{animName = "leftSection2", inputName = "leftSection2Fold", workAreas = {2}, width = 3.497}, 
											{animName = "leftSection3", inputName = "leftSection3Fold", workAreas = {3}, width = 6.029}, 
											{animName = "leftSection4", inputName = "leftSection4Fold", workAreas = {4}, width = 5.103}, 
											{animName = "middleSection", inputName = "middleSectionFold", workAreas = {5}, width = 1.726}, 
											{animName = "rightSection4", inputName = "rightSection4Fold", workAreas = {6}, width = 5.103}, 
											{animName = "rightSection3", inputName = "rightSection3Fold", workAreas = {7}, width = 6.029}, 
											{animName = "rightSection2", inputName = "rightSection2Fold", workAreas = {8}, width = 3.497}, 
											{animName = "rightSection1", inputName = "rightSection1Fold", workAreas = {9}, width = 2.508}};
		local testAreasParentIndex = Utils.indexToObject(self.components, "0>0");
		local testAreaData = {{sx = 18, sy = 0, sz = -1.431, wx = 18, wy = 0, wz = -2.831, hx = 15.492, hy = 0, hz = -1.431},						-- testArea left 1
											{sx = 15.492, sy = 0, sz = -1.431, wx = 15.492, wy = 0, wz = -2.831, hx = 11.995, hy = 0, hz = -1.431},		-- testArea left 2
											{sx = 11.995, sy = 0, sz = -1.431, wx = 11.995, wy = 0, wz = -2.831, hx = 5.966, hy = 0, hz = -1.431},			-- testArea left 3
											{sx = 5.966, sy = 0, sz = -1.431, wx = 5.966, wy = 0, wz = -2.831, hx = 0.863, hy = 0, hz = -1.431},				-- testArea left 4
											{sx = 0.863, sy = 0, sz = -1.97, wx = 0.863, wy = 0, wz = -3.37, hx = -0.863, hy = 0, hz = -1.97},					-- testArea middle
											{sx = -0.863, sy = 0, sz = -1.431, wx = -0.863, wy = 0, wz = -2.831, hx = -5.966, hy = 0, hz = -1.431},			-- testArea right 4
											{sx = -5.966, sy = 0, sz = -1.431, wx = -5.966, wy = 0, wz = -2.831, hx = -11.995, hy = 0, hz = -1.431},		-- testArea right 3
											{sx = -11.995, sy = 0, sz = -1.431, wx = -11.995, wy = 0, wz = -2.831, hx = -15.492, hy = 0, hz = -1.431},	-- testArea right 2
											{sx = -15.492, sy = 0, sz = -1.431, wx = -15.492, wy = 0, wz = -2.831, hx = -18, hy = 0, hz = -1.431}};			-- testArea right 1						
											
		for index, data in pairs(sectionData) do
			local testArea = nil;
			if testAreaData[index] ~= nil then
				local tAData = testAreaData[index];
				testArea = self:createTestArea(tAData.sx, tAData.sy, tAData.sz, tAData.wx, tAData.wy, tAData.wz, tAData.hx, tAData.hy, tAData.hz, testAreasParentIndex);
			end;
			table.insert(self.sections, index, self:createSection(data.animName, data.inputName, data.workAreas, data.width, testArea));
		end;
		
		self:createSCGUI();
	end;
end;

function KuhnDLCAddin:postLoad(savegame)
	if self.configFileName == getUserProfileAppPath().."pdlc/kuhnPack/vehicles/tools/kuhn/kuhnDeltis1302.xml" or self.configFileName == getUserProfileAppPath().."pdlc/kuhnPack/vehicles/trailers/kuhn/kuhnMetris4102.xml" then
		self.sectionControlActive = true;	
	end;
end;

function KuhnDLCAddin:delete()
end;

function KuhnDLCAddin:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KuhnDLCAddin:keyEvent(unicode, sym, modifier, isDown)
end;

function KuhnDLCAddin:update(dt)
end;

function KuhnDLCAddin:draw()
end;

function KuhnDLCAddin:createWorkArea(startX, startY, startZ, widthX, widthY, widthZ, heightX, heightY, heightZ, parentNode)
	local workArea = {};
	
	local start = createTransformGroup("start");
	link(parentNode,  start);
	local sx, sy,sz = localToWorld(start, startX, startY, startZ);
	setTranslation(start, sx, sy, sz);
	workArea.start = start;
	
	local width = createTransformGroup("width");
	link(parentNode,  width);
	local wx, wy,wz = localToWorld(width, widthX, widthY, widthZ);
	setTranslation(width, wx, wy, wz);
	workArea.width = width;
	
	local height = createTransformGroup("height");
	link(parentNode,  height);
	local hx, hy,hz = localToWorld(height, heightX, heightY, heightZ);
	setTranslation(height, hx, hy, hz);
	workArea.height = height;
	
	workArea.disableBackwards = false;
	workArea.type = WorkArea.AREATYPE_SPRAYER;
	workArea.foldLimitedOuterRange = false;
	workArea.foldMinLimit  = 0;
	workArea.foldMaxLimit  = 1;
	
	return workArea;
end;

function KuhnDLCAddin:createTestArea(startX, startY, startZ, widthX, widthY, widthZ, heightX, heightY, heightZ, parentNode)
	local testArea = {};
	
	local start = createTransformGroup("start");
	link(parentNode,  start);
	local sx, sy,sz = localToWorld(start, startX, startY, startZ);
	setTranslation(start, sx, sy, sz);
	testArea.start = start;
	
	local width = createTransformGroup("width");
	link(parentNode,  width);
	local wx, wy,wz = localToWorld(width, widthX, widthY, widthZ);
	setTranslation(width, wx, wy, wz);
	testArea.width = width;
	
	local height = createTransformGroup("height");
	link(parentNode,  height);
	local hx, hy,hz = localToWorld(height, heightX, heightY, heightZ);
	setTranslation(height, hx, hy, hz);
	testArea.height = height;
	
	return testArea;
end;

function KuhnDLCAddin:createSection(animName, inputName, workAreas, width, testArea)
	local section = {};
	
	if animName ~= nil or workArea ~= nil then
		section.animName = animName;
		section.inputName = inputName;
		section.workAreas = workAreas;
		section.width = width;
		section.activated = true;
		
		if testArea ~= nil then
			if testArea.start ~= nil and testArea.width ~= nil and testArea.height ~= nil then
				section.testAreaStart = testArea.start;
				section.testAreaWidth = testArea.width;
				section.testAreaHeight = testArea.height;
			end;
		end;
	end;
	
	if inputName ~= nil then
		local modDesc =  loadXMLFile("modDesc", KuhnDLCAddin.modDir.."modDesc.xml");
		local a=0;
		while true do
			local key = string.format("modDesc.l10n.text(%d)", a);
			if not hasXMLProperty(modDesc, key) then
				break;
			end;
			
			local name = getXMLString(modDesc, key.."#name");
			if name == string.format("input_%s", inputName) then
				g_i18n.globalI18N.texts[name] = getXMLString(modDesc, key.."."..g_languageShort);
				break;
			end;
			
			a=a+1;
		end;
	end;
	
	return section;
end;

function KuhnDLCAddin:createAnimation(name, parts)
	if name ~= nil then
		local animation = {};
		
		animation.name = name
		animation.parts = {};
		animation.currentTime  = 0;
		animation.currentSpeed  = 1;
		animation.looping  = false;
		
		for _, index in pairs(parts) do
			local part = {};
			part.node = Utils.indexToObject(self.components, index);
			part.startTime = 0.1 * 1000;
			part.duration = 0*1000;
			part.direction = 0;
			if part.node ~= nil then
				part.startScale  = Utils.getVectorNFromString("1 1 1", 3);
				part.endScale  = Utils.getVectorNFromString("0 0 0", 3);
			end;
			
			table.insert(animation.parts, part);
		end;
		
		animation.partsReverse = {};
		for _, part in ipairs(animation.parts) do
			table.insert(animation.partsReverse, part);
		end;
		table.sort(animation.parts, AnimatedVehicle.animPartSorter);
		table.sort(animation.partsReverse, AnimatedVehicle.animPartSorterReverse);
		
		AnimatedVehicle.initializeParts(self, animation);
		
		animation.currentPartIndex = 1;
		animation.duration = 0.1*1000;
		
		self.animations[name] = animation;
	end;
end;